<template>
    <div>
        <q-table ref="table" :columns="columns" :rows="data" @request="syncData" :visible-columns="visibleColumns">

            <!--table top section -->
            <template v-slot:top>
                <div class="row full-width q-gutter-y-sm justify-between items-center">
                    <q-btn-group unelevated style="background-color: #ffffff70" v-if="!listView" class="text-grey-8"
                        size="md">
                        <q-btn size="md" icon="print" :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined"
                            @click="printData" />
                        <q-btn size="md" icon="refresh" @click="$refs.table.requestServerInteraction()"
                            :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined" />
                        <q-btn size="md" icon="mdi-pin" :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined"
                            :color="stickyColumns ? this.$store.state.app.color + '-7' : undefined"
                            @click="stickyColumns = !stickyColumns" />
                        <q-btn size="md" icon="fullscreen" :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined"
                            :color="fullscreenMode ? this.$store.state.app.color + '-7' : undefined"
                            @click="fullscreenMode = !fullscreenMode" />
                        <q-btn size="md" icon="mdi-file-excel"
                            :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined" />
                        <q-btn size="md" :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined"
                            :color="perColumnSearch ? this.$store.state.app.color + '-7' : undefined" icon="search"
                            @click="perColumnSearch = !perColumnSearch" />
                        <q-btn size="md" icon="settings" :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined" />
                        <q-btn size="md" :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined"
                            :color="gridMode ? this.$store.state.app.color + '-7' : undefined" icon="apps"
                            @click="gridMode = !gridMode" />
                    </q-btn-group>
                    <q-input v-if="!listView" dense :text-color="$q.dark.isActive ? 'blue-grey-1' : undefined"
                        :color="tableColor" v-model="filter"
                        @update:model-value="(val) => $refs.table.requestServerInteraction()" placeholder="Search" />
                </div>

                <!-- date Range Filter -->
                <div class="row full-width justify-start items-center">
                    <div class="row q-gutter-sm items-center justify-start" v-if="dateColumns.length > 0">
                        <div class="column q-gutter-sm justify-center items-start">
                            <div v-for="drParam, index in dataRange_" :key="index" class="row q-gutter-sm items-center">
                                <q-select outlined hide-bottom-space dense :color="tableColor" :options="dateColumns"
                                    v-model="drParam.column" style="min-width:150px">
                                </q-select>
                                <date-combo v-model="drParam.date"></date-combo>
                            </div>

                        </div>
                        <div class="row q-gutter-sm items-center justify-start">
                            <q-btn size="md" :color="this.$store.state.app.color + '-7'" dense flat icon="filter_alt"
                                @click="$refs.table.requestServerInteraction()"></q-btn>
                            <q-btn size="md" color="red" dense flat icon="close" @click="resetDateRangeFilter">
                            </q-btn>
                        </div>
                    </div>
                    <!-- prefilters-->
                    <div class="row q-gutter-sm col" v-if="preFilters?.length > 0">
                        <q-select dense style="min-width:150px" v-for="(preFilter, index) in preFilters"
                            :label="preFilter.fieldName" :key="index" />
                    </div>
                </div>
            </template>

            <!--custom header -->
            <template v-slot:header-cell="props">
                <q-th :props="props">
                    <div class="text-center" style="white-space: pre-wrap;">{{ props.col.label }}</div>
                    <q-input hide-bottom-space :model-value="props.col.filter"
                        @update:model-value="(value) => changeColumnFilter(props, value)" :type="columnType(props)"
                        v-if="perColumnSearch && props.col.field !== null" class="no-margin no-padding" color="white"
                        dense></q-input>
                </q-th>
            </template>
            <!--init sekelton -->
            <template v-slot:header v-if="!init">
                <q-markup-table :class="markupTableClass">
                    <thead>
                        <tr>
                            <th v-for="n in 10" :key="n" class="text-left" style="width: 150px">
                                <q-skeleton animation="blink" type="text" />
                            </th>
                        </tr>
                    </thead>
                    <tbody>
                        <tr v-for="n in 10" :key="n">
                            <td v-for="n in 10" :key="n" class="text-left">
                                <q-skeleton animation="blink" type="text" width="85px" />
                            </td>
                        </tr>
                    </tbody>
                </q-markup-table>
            </template>

            <!--No Data Custom -->
            <template v-slot:no-data v-if="data.length === 0 && this.init">
                <div class="row justify-center full-width" style="min-height:300px">
                    <h3 class="self-center">No Data</h3>
                </div>
            </template>

            <!--Serial Number Column -->
            <template v-slot:body-cell-tableSlNo="props">
                <q-td :props="props">{{ props.rowIndex + 1 }}</q-td>
            </template>

            <!--Status Column-->
            <template v-slot:body-cell-Status="props">
                <q-td :props="props">
                    <q-badge color="green" v-if="props.row.status == 1">Active</q-badge>
                    <q-badge color="red" v-else>In-Active</q-badge>
                </q-td>
            </template>

            <!--Control Column-->
            <template v-slot:body-cell-tableControl="props">
                <q-td>
                    <slot name="control" :row="props.row"></slot>
                </q-td>
            </template>

            <!--value formatting-->
            <template v-slot:body-cell="props">
                <q-td :props="props">

                    <template v-if="isJSON(props.value)">
                        <q-btn dense @click="$q.dialog({ message: props.value })" size="sm" color="primary">view</q-btn>
                    </template>
                    <template v-else-if="(props.col.fieldObject.type === 'tinyint')">
                        <q-checkbox dense size="sm" color="primary" disable v-model="props.value" :true-value="1"
                            :false-value="0" />
                    </template>
                    <template
                        v-else-if="props.col.type == 'text' || props.col.type == 'longtext' || (typeof props.value === 'string' && props.value.length > 20)">
                        <div class="row no-wrap justify-between">
                            <div @click="$q.dialog({ message: props.value })" class="ellipsis col"
                                style="max-width:250px">
                                {{ props.value }}
                                <q-tooltip>{{ props.value }}</q-tooltip>
                            </div>
                            <div v-if="props.col.linkedModule !== undefined"
                                @click="openRelatedModule(props.col.linkedModule, props.row[props.col.linkedModule.fk])">
                                <q-icon size="xs" name="open_in_new" />
                            </div>
                        </div>
                    </template>
                    <template v-else>
                        <div class="row no-wrap justify-between">
                            <div>{{ props.value }}</div>
                            <div v-if="props.col.linkedModule !== undefined"
                                @click="openRelatedModule(props.col.linkedModule, props.row[props.col.linkedModule.fk])">
                                <q-icon size="xs" name="open_in_new" />
                            </div>
                        </div>
                    </template>
                </q-td>
            </template>


        </q-table>
    </div>
</template>

<script>
const { date } = Quasar
export default {
    props: ['module', 'tableName', 'dataRange', 'extraPostData'],
    components: {
        bTable: Vue.defineAsyncComponent(() => import('components/widgets/b-table.vue')),
        DateCombo: Vue.defineAsyncComponent(() => import('components/widgets/date-combo.vue'))
    },
    watch: {
        dataRange: function (val) {
            this.dataRange_ = JSON.parse(JSON.stringify(val))
        }
    },
    data() {
        return {
            fields: [],
            perColumnSearch: false,
            preFilters: [],
            listView: false,
            init: false,
            filterFields: [],
            data: [],
            dataRange_: [{
                column: null,
                date: {
                    from: null,
                    to: null
                }
            }]
        }
    },

    computed: {
        columns() {
            return this.fields.map((field) => {
                return {
                    label: field.label !== undefined ? field.label : field.name,
                    field: field.name,
                    column: field.name,
                    name: field.name,
                    type: '',
                    fieldObject: {
                        type: field.type
                    }
                }
            })
        },
        visibleColumns() {
            return this.fields.map((field) => {
                return (field.name.toLowerCase() !== 'id' && field.name.substr(-3).toLowerCase() !== '_id') ? field.name : false
            })
        },
        dateColumns() {

            return this.fields.filter((field) => {
                return field.type == 'datetime' || field.type == 'timestamp' || field.type == 'date'
            }).map((field) => {
                return field.name
            })

        }
    },
    methods: {
        getTableDef() {
            const self = this
            axios.post('welcome/build_table/', new URLSearchParams({
                Table_Name: this.tableName,
                Module_Name: this.module.Module_Name
            })).then(({ data }) => {
                let columns = []
                columns.push({ label: 'Si No', name: 'tableSlNo', type: '', length: 0 })
                for (const key in data.datatype.result.fields) {
                    let field = data.datatype.result.fields[key]
                    columns.push(field)
                }
                columns.push({ label: '', name: 'tableControl', type: '', length: 0 })
                self.fields = columns
                self.preFilters = []
                for (const field in data.datatype.result.filter_fields) {
                    self.preFilters.push({ fieldName: field, type: data.datatype.result.filter_fields[field].type })
                }

            })
        },
        syncData() {
            const self = this
            const postData = {
                table_name: this.tableName,
                module_name: this.module.Module_Name
            }
            let datefilters = []
            for (const drParam of this.dataRange_) {
                if (drParam.column != null && drParam.date.from !== null && drParam.date.to !== null) {
                    datefilters.push({
                        field: drParam.column,
                        from_date: drParam.date.from,
                        to_date: drParam.date.to
                    })
                }
            }
            console.log(datefilters)
            postData.datefilters = datefilters

            console.log(postData)
            if (this.extraPostData !== undefined) {
                Object.assign(postData, this.extraPostData)
            }


            axios.post('welcome/datatables/', objectToFormData(postData)).then((response) => {
                self.init = true
                self.data = response.data.datatype.result.data
            })
        },
        resetDateRangeFilter() {
            this.dataRange_ = [{
                column: null,
                date: {
                    from: null,
                    to: null
                }
            }]
            this.$refs.table.requestServerInteraction()
        },
        isJSON: function (str) {
            try {
                const object = JSON.parse(str)
                if (typeof object === 'object' && object !== null) {
                    return true
                }
            } catch (e) {
                return false
            }
            return false
        },
    },
    created() {
        this.getTableDef()
        this.syncData()
    }
}
</script>
